@@ -1,4 +1,4 @@
-EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, solnp_args)
+EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, solnp_args, control)
 {
     verbose <- list$verbose
     lrPars <- list$lrPars
@@ -115,6 +115,9 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
         LBOUND[LBOUND == -Inf] <- -1e10
         UBOUND[UBOUND == Inf] <- 1e10
     }
+    if(Moptim == 'NR' && sum(est) > 300L && list$message)
+        message('NR optimizer should not be used for models with a large number of parameters.
+                Use the optimizer = \'BFGS\' or \'nlminb\' instead.')
     EMhistory <- matrix(NA, NCYCLES+1L, length(longpars))
     EMhistory[1L,] <- longpars
     ANY.PRIOR <- rep(FALSE, ngroups)
@@ -211,7 +214,8 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                               PrepList=PrepList, L=L, UBOUND=UBOUND, LBOUND=LBOUND, Moptim=Moptim,
                               BFACTOR=BFACTOR, nfact=nfact, Thetabetween=Thetabetween,
                               rlist=rlist, constrain=constrain, DERIV=DERIV, Mrate=Mrate,
-                              TOL=list$MSTEPTOL, solnp_args=solnp_args, full=full, lrPars=lrPars)
+                              TOL=list$MSTEPTOL, solnp_args=solnp_args, full=full, lrPars=lrPars,
+                              control=control)
             EMhistory[cycles+1L,] <- longpars
             if(verbose)
                 cat(sprintf('\rIteration: %d, Log-Lik: %.3f, Max-Change: %.5f',
@@ -253,7 +257,7 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                             Elist <- Estep(pars=pars, Data=Data, Theta=Theta, prior=prior, Prior=Prior,
                                            Priorbetween=Priorbetween, specific=specific, sitems=sitems,
                                            ngroups=ngroups, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
-                                           BFACTOR=BFACTOR, rlist=rlist)
+                                           BFACTOR=BFACTOR, rlist=rlist, full=full)
                             if(Elist$LL <= collectLL[cycles]){
                                 accel <- (accel - 1) / 2
                                 count <- count + 1L
