@@ -62,6 +62,11 @@
 #' itemplot(pmod, 3, type = 'score')
 #' itemplot(pmod, 3, type = 'infotrace')
 #'
+#' # use the directlabels package to put labels on tracelines
+#' library(directlabels)
+#' plt <- itemplot(pmod, 3)
+#' direct.label(plt, 'top.points')
+#'
 #' # uncomment to run interactive shiny applet
 #' # itemplot(shiny = TRUE)
 #'     }
@@ -72,8 +77,10 @@ itemplot <- function(object, item, type = 'trace', degrees = 45, CE = FALSE, CEa
                      theta_lim = c(-6,6), shiny = FALSE, ...){
     if(shiny){
         if(requireNamespace("shiny", quietly = TRUE))
-            shiny::runApp(shinyItemplot())
+            shiny::runApp(shinyItemplot(), ...)
     }
+    if(missing(object)) missingMsg('object')
+    if(missing(item)) missingMsg('item')
     if(is(object, 'DiscreteClass'))
         stop('Discrete latent structures not yet supported')
     if(is.list(object)) inames <- colnames(object[[1]]@Data$data)
