@@ -3,16 +3,16 @@
 #' Compute a Wald test given an \code{L} vector or matrix of numeric contrasts. Requires that the
 #' model information matrix be computed (including \code{SE = TRUE} when using the EM method). Use
 #' \code{wald(model)} to observe how the information matrix columns are named, especially if
-#' the estimated model contains constrained parameters (e.g., 1PL). 
+#' the estimated model contains constrained parameters (e.g., 1PL).
 #'
 #'
 #' @aliases wald
-#' @param L a coefficient matrix with dimensions nconstrasts x npars. 
+#' @param L a coefficient matrix with dimensions nconstrasts x npars.
 #'   Omitting this value will return the column names of the
 #'   information matrix used to identify the (potentially constrained) parameters
 #' @param object estimated object from \code{mirt}, \code{bfactor},
 #'   \code{multipleGroup}, \code{mixedmirt}, or \code{mdirt}
-#' @param C a constant vector of population parameters to be compared along side L, where 
+#' @param C a constant vector of population parameters to be compared along side L, where
 #'   \code{length(C) == ncol(L)}. By default a vector of 0's is constructed
 #' @keywords wald
 #' @export wald
@@ -24,7 +24,7 @@
 #' mod <- mirt(data, 1, SE = TRUE)
 #' coef(mod)
 #'
-#' # see how the information matrix relates to estimated parameters, and how it lines up 
+#' # see how the information matrix relates to estimated parameters, and how it lines up
 #' #   with the parameter index
 #' (infonames <- wald(mod))
 #' index <- mod2values(mod)
@@ -45,13 +45,14 @@
 #' #logLiklihood tests (requires estimating a new model)
 #' cmodel <- mirt.model('theta = 1-5
 #'                       CONSTRAIN = (1,2, a1), (4,5, a1)')
-#' mod2 <- mirt(data, cmodel) 
-#' #or, eqivalently                     
+#' mod2 <- mirt(data, cmodel)
+#' #or, eqivalently
 #' #mod2 <- mirt(data, 1, constrain = list(c(1,5), c(13,17)))
 #' anova(mod2, mod)
-#' 
+#'
 #' }
 wald <- function(object, L, C = 0){
+    if(missing(object)) missingMsg('object')
     if(all(dim(object@information) == c(1,1)))
         if(object@information[1,1] == 0L)
             stop('No information matrix has been calculated for the model')
