@@ -2,12 +2,11 @@
 #define _ESTEP_H
 
 void _Estep(vector<double> &, vector<double> &, const vector<double> &,
-    const vector<double> &, const IntegerMatrix &, const NumericMatrix &,
-    const int &);
+    const vector<double> &, const IntegerMatrix &, const NumericMatrix &);
 
 void _Estepbfactor(vector<double> &, vector<double> &, vector<double> &, 
     const NumericMatrix &, const vector<double> &, const vector<double> &, 
-    const vector<double> &, const int &, const IntegerMatrix &, const IntegerMatrix &,
+    const vector<double> &, const IntegerMatrix &, const IntegerMatrix &,
     const vector<double> &);
 
 #endif
